import { hash } from "ohash";
import { defineEncodeDataAttribute } from "@sanity/core-loader/encode-data-attribute";
import { onScopeDispose, reactive, ref } from "vue";
import useSanityClient from "../utils/useSanityClient.js";
import { useAsyncData, useLazyAsyncData } from "#imports";
export const useSanityQuery = (query, _params = {}, _options = {}, lazy = false) => {
  const { client: clientType, perspective: _perspective, ...options } = _options;
  const client = clientType ? useSanityClient(clientType) : useSanityClient();
  const hasQueryStore = "queryStore" in client && client.queryStore !== null;
  const perspective = _perspective || (hasQueryStore ? "previewDrafts" : "published");
  const reactiveParams = _params ? reactive(_params) : void 0;
  if (reactiveParams) {
    options.watch ||= [];
    options.watch.push(reactiveParams);
  }
  const key = "sanity-" + hash(query + (reactiveParams ? JSON.stringify(reactiveParams) : ""));
  const data = ref(null);
  const sourceMap = ref(null);
  const encodeDataAttribute = ref(() => {
  });
  function updateResult(newData, newSourceMap) {
    data.value = newData;
    sourceMap.value = newSourceMap || null;
    if (client.config.visualEditing) {
      encodeDataAttribute.value = defineEncodeDataAttribute(newData, newSourceMap, client.config.visualEditing?.studioUrl);
    }
  }
  let fetchFunc = async () => {
    return { data: await client.fetch(query, reactiveParams || {}, { perspective }) };
  };
  if (hasQueryStore) {
    let setupFetcher = function(cb) {
      unsubscribe();
      const deepClonedParams = _params ? JSON.parse(JSON.stringify(_params)) : void 0;
      const fetcher = defaultClient.queryStore.createFetcherStore(query, deepClonedParams, void 0);
      unsubscribe = fetcher.subscribe((newSnapshot) => {
        if (newSnapshot.data) {
          updateResult(newSnapshot.data, newSnapshot.sourceMap);
          cb?.(newSnapshot);
        }
      });
    };
    const defaultClient = client;
    let unsubscribe = () => {
    };
    const proxyClient = {
      fetch: (query2, params, options2) => $fetch(defaultClient.config.visualEditing.proxyEndpoint || "/_sanity/fetch", {
        method: "POST",
        body: { query: query2, params, options: options2 }
      })
    };
    fetchFunc = async () => {
      const currentClient = import.meta.server ? defaultClient.queryStore.unstable__serverClient.instance || defaultClient.client : proxyClient;
      const { result: fetchData, resultSourceMap: sourceMap2 } = await currentClient.fetch(query, reactiveParams || {}, {
        perspective,
        filterResponse: false,
        resultSourceMap: "withKeyArraySelector"
      });
      return sourceMap2 ? { data: fetchData, sourceMap: sourceMap2 } : { data: fetchData };
    };
    if (import.meta.client) setupFetcher();
    onScopeDispose(unsubscribe);
  }
  const pendingData = lazy ? useLazyAsyncData(key, fetchFunc, options) : useAsyncData(key, fetchFunc, options);
  return Object.assign(new Promise((resolve) => pendingData.then(({ data: { value } }) => {
    updateResult(value.data, value.sourceMap);
    resolve({ ...pendingData, data, sourceMap, encodeDataAttribute });
  })), { ...pendingData, data, sourceMap, encodeDataAttribute });
};
